@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of RhythmDB query creation dialog
  *
  *  Copyright (C) 2003, 2004 Colin Walters <walters@gnome.org>
  *
@@ -41,12 +39,11 @@
 #include "rb-dialog.h"
 #include "rb-debug.h"
 #include "rb-preferences.h"
-#include "rb-glade-helpers.h"
+#include "rb-builder-helpers.h"
 #include "rb-util.h"
 
 static void rb_query_creator_class_init (RBQueryCreatorClass *klass);
-static GObject *rb_query_creator_constructor (GType type, guint n_construct_properties,
-					      GObjectConstructParam *construct_properties);
+static void rb_query_creator_constructed (GObject *object);
 static void rb_query_creator_dispose (GObject *object);
 static void rb_query_creator_set_property (GObject *object,
 					      guint prop_id,
@@ -120,8 +117,8 @@ G_DEFINE_TYPE (RBQueryCreator, rb_query_creator, GTK_TYPE_DIALOG)
  * of entries, the total duration, or the total file size; and also the
  * order in which the results are to be sorted.
  *
- * The structure of the query creator is defined in the glade file 
- * create-playlist.xml.
+ * The structure of the query creator is defined in the builder file
+ * create-playlist.ui.
  */
 
 enum
@@ -137,7 +134,7 @@ rb_query_creator_class_init (RBQueryCreatorClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->dispose = rb_query_creator_dispose;
-	object_class->constructor = rb_query_creator_constructor;
+	object_class->constructed = rb_query_creator_constructed;
 	object_class->set_property = rb_query_creator_set_property;
 	object_class->get_property = rb_query_creator_get_property;
 
@@ -176,18 +173,18 @@ rb_query_creator_init (RBQueryCreator *creator)
 
 }
 
-static GObject *
-rb_query_creator_constructor (GType type,
-                              guint n_construct_properties,
-			      GObjectConstructParam *construct_properties)
+static void
+rb_query_creator_constructed (GObject *object)
 {
 	RBQueryCreatorPrivate *priv;
 	RBQueryCreator *creator;
-	GladeXML *xml;
 	GtkWidget *mainbox;
+	GtkWidget *content_area;
+	GtkBuilder *builder;
 
-	creator = RB_QUERY_CREATOR (G_OBJECT_CLASS (rb_query_creator_parent_class)
-			->constructor (type, n_construct_properties, construct_properties));
+	RB_CHAIN_GOBJECT_METHOD (rb_query_creator_parent_class, constructed, object);
+
+	creator = RB_QUERY_CREATOR (object);
 	priv = QUERY_CREATOR_GET_PRIVATE (creator);
 
 	if (priv->creating) {
@@ -215,22 +212,23 @@ rb_query_creator_constructor (GType type,
 	else
 		gtk_window_set_title (GTK_WINDOW (creator), _("Edit Automatic Playlist"));
 
+	content_area = gtk_dialog_get_content_area (GTK_DIALOG (creator));
+
 	gtk_container_set_border_width (GTK_CONTAINER (creator), 5);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (creator)->vbox), 2);
-	gtk_dialog_set_has_separator (GTK_DIALOG (creator), FALSE);
+	gtk_box_set_spacing (GTK_BOX (content_area), 2);
 
-	xml = rb_glade_xml_new ("create-playlist.glade",
-				"main_vbox",
-				creator);
+	builder = rb_builder_load ("create-playlist.ui", creator);
 
-	priv->disjunction_check = GTK_WIDGET (glade_xml_get_widget (xml, "disjunctionCheck"));
-	priv->limit_check = GTK_WIDGET (glade_xml_get_widget (xml, "limitCheck"));
-	priv->limit_entry = GTK_WIDGET (glade_xml_get_widget (xml, "limitEntry"));
-	priv->limit_option = GTK_WIDGET (glade_xml_get_widget (xml, "limitOption"));
-	priv->addbutton = GTK_WIDGET (glade_xml_get_widget (xml, "addButton"));
-	priv->sort_label = GTK_WIDGET (glade_xml_get_widget (xml, "sortLabel"));
-	priv->sort_menu = GTK_WIDGET (glade_xml_get_widget (xml, "sortMenu"));
-	priv->sort_desc = GTK_WIDGET (glade_xml_get_widget (xml, "sortDesc"));
+	priv->disjunction_check = GTK_WIDGET (gtk_builder_get_object (builder, "disjunctionCheck"));
+	priv->limit_check = GTK_WIDGET (gtk_builder_get_object (builder, "limitCheck"));
+	priv->limit_entry = GTK_WIDGET (gtk_builder_get_object (builder, "limitEntry"));
+	priv->limit_option = GTK_WIDGET (gtk_builder_get_object (builder, "limitOption"));
+	priv->addbutton = GTK_WIDGET (gtk_builder_get_object (builder, "addButton"));
+	priv->sort_label = GTK_WIDGET (gtk_builder_get_object (builder, "sortLabel"));
+	priv->sort_menu = GTK_WIDGET (gtk_builder_get_object (builder, "sortMenu"));
+	priv->sort_desc = GTK_WIDGET (gtk_builder_get_object (builder, "sortDesc"));
+
+	gtk_combo_box_set_active (GTK_COMBO_BOX (priv->limit_option), 0);
 
 	g_signal_connect_object (G_OBJECT (priv->limit_check), "toggled", G_CALLBACK (limit_toggled_cb),
 				 creator, 0);
@@ -242,17 +240,15 @@ rb_query_creator_constructor (GType type,
 
 	setup_sort_option_menu (creator, priv->sort_menu, sort_options, num_sort_options);
 
-	priv->vbox = GTK_BOX (glade_xml_get_widget (xml, "sub_vbox"));
+	priv->vbox = GTK_BOX (gtk_builder_get_object (builder, "sub_vbox"));
 	if (priv->creating)
 		append_row (creator);
 
-	mainbox = glade_xml_get_widget (xml, "main_vbox");
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (creator)->vbox), mainbox, FALSE, FALSE, 0);
+	mainbox = GTK_WIDGET (gtk_builder_get_object (builder, "complex-playlist-creator"));
+	gtk_box_pack_start (GTK_BOX (content_area), mainbox, FALSE, FALSE, 0);
 	gtk_widget_show_all (GTK_WIDGET (creator));
 
-	g_object_unref (xml);
-
-	return G_OBJECT (creator);
+	g_object_unref (builder);
 }
 
 static void
@@ -265,22 +261,29 @@ rb_query_creator_dispose (GObject *object)
 	priv = QUERY_CREATOR_GET_PRIVATE (object);
 	g_return_if_fail (priv != NULL);
 
-	if (priv->property_size_group != NULL)
-		g_object_unref (G_OBJECT (priv->property_size_group));
-	priv->property_size_group = NULL;
-	if (priv->criteria_size_group != NULL)
-		g_object_unref (G_OBJECT (priv->criteria_size_group));
-	priv->criteria_size_group = NULL;
-	if (priv->entry_size_group != NULL)
-		g_object_unref (G_OBJECT (priv->entry_size_group));
-	priv->entry_size_group = NULL;
-	if (priv->button_size_group != NULL)
-		g_object_unref (G_OBJECT (priv->button_size_group));
-	priv->button_size_group = NULL;
+	if (priv->property_size_group != NULL) {
+		g_object_unref (priv->property_size_group);
+		priv->property_size_group = NULL;
+	}
 
-	if (priv->rows)
+	if (priv->criteria_size_group != NULL) {
+		g_object_unref (priv->criteria_size_group);
+		priv->criteria_size_group = NULL;
+	}
+	if (priv->entry_size_group != NULL) {
+		g_object_unref (priv->entry_size_group);
+		priv->entry_size_group = NULL;
+	}
+
+	if (priv->button_size_group != NULL) {
+		g_object_unref (priv->button_size_group);
+		priv->button_size_group = NULL;
+	}
+
+	if (priv->rows) {
 		g_list_free (priv->rows);
-	priv->rows = NULL;
+		priv->rows = NULL;
+	}
 
 	G_OBJECT_CLASS (rb_query_creator_parent_class)->dispose (object);
 }
@@ -335,6 +338,8 @@ rb_query_creator_get_property (GObject *object,
  * rb_query_creator_new:
  * @db: the #RhythmDB instance
  *
+ * Creates a new query creator widget.
+ *
  * Return value: new query creator widget
  */
 GtkWidget *
@@ -911,7 +916,7 @@ create_property_option_menu (RBQueryCreator *creator,
 
 	combo = gtk_combo_box_new_text ();
 	for (i = 0; i < length; i++) {
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _(options[i].name));
+		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), g_dpgettext2 (NULL, "query-criteria", options[i].name));
 	}
 
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
@@ -966,10 +971,11 @@ setup_sort_option_menu (RBQueryCreator *creator,
 		GtkTreeIter iter;
 
 		gtk_list_store_append (store, &iter);
-		gtk_list_store_set (store, &iter, 0, _(options[i].name), -1);
+		gtk_list_store_set (store, &iter, 0, g_dpgettext2 (NULL, "query-sort", options[i].name), -1);
 	}
 
 	gtk_combo_box_set_model (GTK_COMBO_BOX (option_menu), GTK_TREE_MODEL (store));
+	gtk_combo_box_set_active (GTK_COMBO_BOX (option_menu), 0);
 
 	g_signal_connect_object (G_OBJECT (option_menu), "changed",
 				 G_CALLBACK (sort_option_menu_changed), creator, 0);
